/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ODDINFO_H
#define DPC_IMPL_TYPE_ODDINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_boolean.h"
#include "impl_type_uint8.h"
#include "impl_type_double.h"

namespace dpc {
struct ODDInfo {
    ::Boolean valid{ false };
    ::UInt8 lane_id{ 0U };
    ::UInt8 type{ 0U };
    ::UInt8 sub_type{ 0U };
    ::Double start_offset{ 0.0 };
    ::Double end_offset{ 0.0 };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(valid);
        fun(lane_id);
        fun(type);
        fun(sub_type);
        fun(start_offset);
        fun(end_offset);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(valid);
        fun(lane_id);
        fun(type);
        fun(sub_type);
        fun(start_offset);
        fun(end_offset);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("valid", valid);
        fun("lane_id", lane_id);
        fun("type", type);
        fun("sub_type", sub_type);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("valid", valid);
        fun("lane_id", lane_id);
        fun("type", type);
        fun("sub_type", sub_type);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
    }

    friend bool operator==(const ::dpc::ODDInfo& lhs, const ::dpc::ODDInfo& rhs) noexcept
    {
        return (lhs.valid == rhs.valid) && (lhs.lane_id == rhs.lane_id) && (lhs.type == rhs.type) && (lhs.sub_type == rhs.sub_type) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ODDINFO_H
